//+------------------------------------------------------------------+
//|                                                    kama macd.mq4 |
//|                                                                  |
//| Adaptive moving average (AMA) Perry Kaufman variation            |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  DeepSkyBlue
#property indicator_color2  Red
#property indicator_width1  2
#property indicator_level1  0
#property indicator_levelcolor DimGray

//
//
//
//
//

extern int    MacdFast      = 12;    
extern int    MacdSlow      = 26;    
extern int    Signal        =  9;
extern int    KamaPrice     = PRICE_CLOSE;
extern int    FastEndPeriod =   2;
extern int    SlowEndPeriod =  30;
extern double Power         = 2.0;

//
//
//
//
//

double macdBuffer[];
double signalBuffer[];
double fastAmaBuffer[];
double slowAmaBuffer[];
double prcBuffer[];
double difBuffer[];
double difsBuffer[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(7);
      SetIndexBuffer(0,macdBuffer);
      SetIndexBuffer(1,signalBuffer);
      SetIndexBuffer(2,fastAmaBuffer);
      SetIndexBuffer(3,slowAmaBuffer);
      SetIndexBuffer(4,prcBuffer);
      SetIndexBuffer(5,difBuffer);
      SetIndexBuffer(6,difsBuffer);
   IndicatorShortName("Kama macd ("+MacdFast+","+MacdSlow+","+Signal+")");         
   return(0);
}
int deinit() {  return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//


int start()
{
   double slowend=2.0/(SlowEndPeriod+1);
   double fastend=2.0/(FastEndPeriod+1);
   int    counted_bars=IndicatorCounted();
   int    limit,i,k;

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);
           
   //
   //
   //
   //
   //

   for(i = limit; i >= 0; i--)
   {
      prcBuffer[i] = iMA(NULL,0,1,0,MODE_SMA,KamaPrice,i);
         if (i>=(Bars-MacdSlow))
         {
            fastAmaBuffer[i] = prcBuffer[i];
            slowAmaBuffer[i] = prcBuffer[i];
            signalBuffer[i]  = 0;
            macdBuffer[i]    = 0;
            continue;
         }
         
      //
      //
      //
      //
      //
         
         double smooth;
         double noise    = 0;
         double effratio = 1;
         double signal   = MathAbs(prcBuffer[i]-prcBuffer[i+MacdFast]);
            difBuffer[i] = MathAbs(prcBuffer[i]-prcBuffer[i+1]);
         
            //
            //
            //
            //
            //
         
            for (k=0; k<MacdFast; k++) noise += difBuffer[i+k];
               if (noise != 0) effratio = signal/noise;  smooth = MathPow(effratio*(fastend-slowend)+slowend,Power);
                  fastAmaBuffer[i] = fastAmaBuffer[i+1]+smooth*(prcBuffer[i]-fastAmaBuffer[i+1]);

            //
            //
            //
            //
            //
            
            noise    = 0;                  
            effratio = 1;                  
            signal   = MathAbs(prcBuffer[i]-prcBuffer[i+MacdSlow]);
            
            for (k=0; k<MacdSlow; k++) noise += difBuffer[i+k];
               if (noise != 0) effratio = signal/noise;  smooth = MathPow(effratio*(fastend-slowend)+slowend,Power);
                  slowAmaBuffer[i] = slowAmaBuffer[i+1]+smooth*(prcBuffer[i]-slowAmaBuffer[i+1]);
                  
            //
            //
            //
            //
            //
                              
         macdBuffer[i] = fastAmaBuffer[i]-slowAmaBuffer[i];

         //
         //
         //
         //
         //

         if (Signal>1)
         {
            difsBuffer[i] = MathAbs(macdBuffer[i]-macdBuffer[i+1]);
               noise    = 0;                  
               effratio = 1;                  
               signal   = MathAbs(macdBuffer[i]-macdBuffer[i+Signal]);
            
               for (k=0; k<Signal; k++) noise += difsBuffer[i+k];
                  if (noise != 0) effratio = signal/noise;  smooth = MathPow(effratio*(fastend-slowend)+slowend,Power);
                     signalBuffer[i] = signalBuffer[i+1]+smooth*(macdBuffer[i]-signalBuffer[i+1]);
         }                     
   }

   //
   //
   //
   //
   //

   return(0);      
}